# Sticky Time Zone v0.9 plugin for Movable Type
# http://www.chipple.net/mt/StickyTZ/
# by Patrick Benny http://www.chipple.net/
# This code cannot be redistributed without permission from the author. 
#
# Version history
#   0.9 - 2008/4/28 - Public preview release

package MT::Plugin::StickyTZ;

use strict;
use base 'MT::Plugin';
use vars qw($VERSION);
use MT::Util qw( encode_html );
$VERSION = '0.9';

my $plugin;
$plugin = MT::Plugin::NotifyReplyee->new({
    name => '<MT_TRANS phrase="Sticky Time Zone">',
    version => $VERSION,
    doc_link => "http://www.chipple.net/mt/StickyTZ/",
    description => '<MT_TRANS phrase="Automatically sets blog time zone when a change is detected in client time.">',
    author_name => '<MT_TRANS phrase="Patrick Benny">',
    author_link => "http://www.chipple.net/",
    l10n_class => 'StickyTZ::L10N',
    system_config_template => 'sysconfig.tmpl',
    config_template => 'config.tmpl',
    settings => new MT::PluginSettings([
        ['stz_enabled',  { Default => '0' }],
    ]),
    registry => {
        applications => {
            cms => {
                methods => {
                    stz_set_tz => \&stz_set_tz,
                },
            },
        },
    },
});
MT->add_plugin($plugin);

MT->add_callback('MT::App::CMS::template_source', 9, $plugin, \&_template);

sub instance { $plugin }


############
### MAIN ###
############

sub stz_set_tz {
  my $app = shift;
  my $blog = MT::Blog->load($app->param('blog_id'));
  $blog->server_offset($app->param('tz'));
  $blog->save();
  $app->redirect($app->param('url')."&stz_changed=1");
}


#################
### UTILITIES ###
#################

sub apply_default_settings {
  my $plugin = shift;
  my ($data, $scope) = @_;
  if ($scope ne 'system') {
    my $sys = $plugin->get_config_obj('system');
    my $sysdata = $sys->data();
    if ($plugin->{settings} && $sysdata) {
      foreach (keys %$sysdata) {
        $data->{$_} = $sysdata->{$_}
          if (!exists $data->{$_}) || (!defined $data->{$_});
      }
    }
  } else {
    $plugin->SUPER::apply_default_settings(@_);
  }
}


################
### TEMPLATE ###
################

# Add options to comment_reply dialog
sub _template {

  my ($cb, $app, $template) = @_;

  return if !$app->blog || !$app->blog->id;

  return if $app->request_method eq 'POST';

  return if $app->cookie_val('mt_stz_quiet') eq '1';

  # Get options from plugin config
  my $cfg_obj = $plugin->get_config_obj('blog:' . $app->blog->id);
  $cfg_obj = $plugin->get_config_obj('system') if $cfg_obj->id eq '';
  if ($cfg_obj->id ne '') {
    my $cfg_data = $cfg_obj->data();
    return if $cfg_data->{'stz_enabled'} ne '1';
  }
  else {
    return;
  }

  my $add = '
<script type="text/javascript">
function stz_f_GetTZName (iTZ) {
  var asTZNames = {
        "13": "<__trans phrase="UTC+13 (New Zealand Daylight Savings Time)" escape="js">",
        "12": "<__trans phrase="UTC+12 (International Date Line East)" escape="js">",
        "11": "<__trans phrase="UTC+11" escape="js">",
        "10": "<__trans phrase="UTC+10 (East Australian Time)" escape="js">",
        "9.5": "<__trans phrase="UTC+9.5 (Central Australian Time)" escape="js">",
        "9": "<__trans phrase="UTC+9 (Japan Time)" escape="js">",
        "8": "<__trans phrase="UTC+8 (China Coast Time)" escape="js">",
        "7": "<__trans phrase="UTC+7 (West Australian Time)" escape="js">",
        "6.5": "<__trans phrase="UTC+6.5 (North Sumatra)" escape="js">",
        "6": "<__trans phrase="UTC+6 (Russian Federation Zone 5)" escape="js">",
        "5.5": "<__trans phrase="UTC+5.5 (Indian)" escape="js">",
        "5": "<__trans phrase="UTC+5 (Russian Federation Zone 4)" escape="js">",
        "4": "<__trans phrase="UTC+4 (Russian Federation Zone 3)" escape="js">",
        "3.5": "<__trans phrase="UTC+3.5 (Iran)" escape="js">",
        "3": "<__trans phrase="UTC+3 (Baghdad Time/Moscow Time)" escape="js">",
        "2": "<__trans phrase="UTC+2 (Eastern Europe Time)" escape="js">",
        "1": "<__trans phrase="UTC+1 (Central European Time)" escape="js">",
        "0": "<__trans phrase="UTC+0 (Universal Time Coordinated)" escape="js">",
        "-1": "<__trans phrase="UTC-1 (West Africa Time)" escape="js">",
        "-2": "<__trans phrase="UTC-2 (Azores Time)" escape="js">",
        "-3": "<__trans phrase="UTC-3 (Atlantic Time)" escape="js">",
        "-3.5": "<__trans phrase="UTC-3.5 (Newfoundland)" escape="js">",
        "-4": "<__trans phrase="UTC-4 (Atlantic Time)" escape="js">",
        "-5": "<__trans phrase="UTC-5 (Eastern Time)" escape="js">",
        "-6": "<__trans phrase="UTC-6 (Central Time)" escape="js">",
        "-7": "<__trans phrase="UTC-7 (Mountain Time)" escape="js">",
        "-8": "<__trans phrase="UTC-8 (Pacific Time)" escape="js">",
        "-9": "<__trans phrase="UTC-9 (Alaskan Time)" escape="js">",
        "-10": "<__trans phrase="UTC-10 (Aleutians-Hawaii Time)" escape="js">",
        "-11": "<__trans phrase="UTC-11 (Nome Time)" escape="js">"
     };
  return asTZNames[iTZ.toString()];
}
function stz_f_SetQuiet () {
  document.cookie = "mt_stz_quiet=1";
}
function stz_f_CheckQuiet () {
  return document.cookie.indexOf("mt_stz_quiet=1") != -1;
}
function stz_f_CheckTZ () {

//  if (stz_f_CheckQuiet()) return;

  var iBlogTZ = '.$app->blog->server_offset.';
  var dServer = new Date("'.MT::Util::offset_time_list(time,$app->blog->id).'");
  var dUser = new Date();
  var iDiffMin = (dServer-dUser)/60000;
  var iDir = iDiffMin < 0 ? 1 : -1;
  iDiffMin = Math.abs(iDiffMin);
  if (iDiffMin >= 20) {
    var iUserTZ = iBlogTZ;
    while (iDiffMin >= 10) {
      iUserTZ += 0.5*iDir;
      iDiffMin -= 30;
    }
    var sUserTZName = stz_f_GetTZName(iUserTZ);
    if (sUserTZName == undefined) {
      alert("Your computer\'s time appears to be in an invalid time zone,\n... or one Movable Type doesn\'t know of!\n\n"+
                   "   Blog TZ: "+stz_f_GetTZName(iBlogTZ)+"\n"+
           "   Server time: "+dServer+"\n"+
           "   User time: "+dUser+"\n"+
//           "   (Diff: "+(dServer-dUser)+")\n"+
           "   User calculated TZ: "+iUserTZ+"\n\n"+
           "(... I will now go quiet.\nTo check your time zone again, you must close your browser once.)");
      stz_f_SetQuiet();
    }
    else if (confirm(
                "Your time zone has changed!\n"+
                "Do you wish to adjust this blog\'s time zone accordingly?\n\n"+
                "   Blog TZ: "+stz_f_GetTZName(iBlogTZ)+"\n"+
                "   Server time: "+dServer+"\n"+
                "   User time: "+dUser+"\n"+
//                "   (Diff: "+(dServer-dUser)+")\n"+
                "   User TZ: "+stz_f_GetTZName(iUserTZ)+"\n\n"+
                "OK = Change blog time zone\n"+
                "Cancel = Leave blog time zone as is (no more check within current session)")) {
      location.href = "'.$app->uri.'?__mode=stz_set_tz&tz="+iUserTZ+"&url="+escape("'.$app->uri.'"+location.search+location.hash);
    }
    else
      stz_f_SetQuiet();
  }

}
stz_f_CheckTZ();
</script>
';

  $$template =~ s|</head>|$add</head>|;

}

sub _param {

  my ($cb, $app, $param) = @_;

  return if !$app->blog || !$app->blog->id;

  return if $app->request_method eq 'POST';

  if ($app->param('stz_changed') eq '1') {
    
    return;
  }

}

1;